# frozen_string_literal: true

require_relative '../environment'

module CodeFormatter
  class Formatter
    attr_reader :config
    attr_reader :path

    def initialize(config, path = Dir.pwd)
      raise TypeError unless config.is_a? Configuration
      raise TypeError unless Dir.exist?(path)
      @config = config
      @path = path
    end

    def format(force_all = false)
      return unless commands_exist?

      files = files_to_format(force_all)
      if files.empty?
        puts "[#{shell_command}]: everything is up-to-date"
      else
        puts "[#{shell_command}]: going to format files:\n#{files.join('\n')}"
        format_files(files)
      end
    end

    def format_files(*)
      raise NotImplementedError
    end

    def shell_command
      raise NotImplementedError
    end

    def files_filter
      raise NotImplementedError
    end

    def commands_exist?
      unless Environment.exist? 'git'
        warn 'warning: failed to find git command'
        return false
      end

      unless Environment.exist? shell_command
        warn "warning: [#{shell_command}] not found (to install it run 'brew install #{shell_command}')"
        return false
      end

      true
    end

    def files_to_format(force_all)
      files = force_all ? all_files : modified_and_staged_files
      files.lines.uniq.map { |f| File.expand_path f.strip }.select { |f| format_file? f }
    end

    def all_files
      `git ls-files -- #{files_filter}`
    end

    def modified_and_staged_files
      modified_and_untracked = `git ls-files --others --exclude-standard -m -- #{files_filter}`
      staged = `git diff --cached --name-only -- #{files_filter}`
      modified_and_untracked + staged
    end

    def format_file?(file)
      return false unless File.file?(file) && File.exist?(file)
      return false if excluded_file?(file)
      included_file?(file)
    end

    def included_file?(file)
      included_files = config.included_files.map { |f| File.expand_path(f, path) }
      included_files.any? { |f| file.start_with?(f) }
    end

    def excluded_file?(file)
      excluded_files = config.excluded_files.map { |f| File.expand_path(f, path) }
      excluded_files.any? { |f| file.start_with?(f) }
    end
  end
end
